package com.zehfernando.ld27.display.scenes {
	import com.zehfernando.display.scenes.AbstractScene;
	import com.zehfernando.display.shapes.Box;
	import com.zehfernando.ld27.application.LD27;
	import com.zehfernando.transitions.ZTween;

	import flash.display.DisplayObject;
	import flash.display.Sprite;
	import flash.events.Event;
	/**
	 * @author zeh fernando
	 */
	public class LD27Scene extends AbstractScene {

		// Constants
		private static const COLOR_FADE_OUT:int = 0x000000;
		private static const TIME_SHOW:Number = 1;
		private static const TIME_HIDE:Number = 1;

		// Instances
		private var cover:Box;
		private var container:Sprite;


		// ================================================================================================================
		// CONSTRUCTOR ----------------------------------------------------------------------------------------------------

		public function LD27Scene() {
			super();

			container = new Sprite();
			super.addChild(container);

			cover = new Box(_width, _height, COLOR_FADE_OUT);
			cover.cacheAsBitmap = true;
			super.addChild(cover);
		}

		// ================================================================================================================
		// INTERNAL INTERFACE ---------------------------------------------------------------------------------------------

		override protected function redrawWidth():void {
			cover.width = _width;
		}

		override protected function redrawHeight():void {
			cover.height = _height;
		}

		override protected function redrawVisibility():void {
			cover.alpha = 1 - _visibility;
			cover.visible = _visibility < 1;
			//visible = _visibility > 0; // This is necessary to cover the starling background
			visible = true;
			alpha = 1;
		}


		// ================================================================================================================
		// EVENT INTERFACE ------------------------------------------------------------------------------------------------

		private function onShow():void {
			// Finished showing
			dispatchEvent(new Event(EVENT_FINISHED_SHOWING));
		}

		private function onHide():void {
			// Finished hiding
			dispatchEvent(new Event(EVENT_FINISHED_HIDING));
		}


		// ================================================================================================================
		// PUBLIC INTERFACE -----------------------------------------------------------------------------------------------

		override public function addChild(child:DisplayObject):DisplayObject {
			return container.addChild(child);
		}

		override public function addChildAt(child:DisplayObject, index:int):DisplayObject {
			return container.addChildAt(child, index);
		}

		override public function removeChild(child:DisplayObject):DisplayObject {
			return container.removeChild(child);
		}

		override public function removeChildAt(index:int):DisplayObject {
			return container.removeChildAt(index);
		}

		override public function removeChildren(beginIndex:int = 0, endIndex:int = 2147483647):void {
			container.removeChildren(beginIndex, endIndex);
		}

		override public function show():void {
			if (LD27.DEBUG_SKIP_SCENE_TRANSITIONS) {
				dispatchEvent(new Event(EVENT_STARTED_SHOWING));
				visibility = 1;
				dispatchEvent(new Event(EVENT_FINISHED_SHOWING));
			} else {
				dispatchEvent(new Event(EVENT_STARTED_SHOWING));
				ZTween.remove(this, "visibility");
				ZTween.add(this, {visibility:1}, {time:TIME_SHOW, onComplete:onShow});
			}
		}

		override public function hide():void {
			if (LD27.DEBUG_SKIP_SCENE_TRANSITIONS) {
				dispatchEvent(new Event(EVENT_STARTED_HIDING));
				visibility = 0;
				dispatchEvent(new Event(EVENT_FINISHED_HIDING));
			} else {
				dispatchEvent(new Event(EVENT_STARTED_HIDING));
				ZTween.remove(this, "visibility");
				ZTween.add(this, {visibility:0}, {time:TIME_HIDE, onComplete:onHide});
			}
		}
	}
}
