package com.zehfernando.ld27.display.scenes.end {
	import com.zehfernando.display.components.text.TextSprite;
	import com.zehfernando.display.components.text.TextSpriteAlign;
	import com.zehfernando.input.binding.GamepadControls;
	import com.zehfernando.input.binding.KeyActionBinder;
	import com.zehfernando.ld27.assets.FontLibrary;
	import com.zehfernando.ld27.display.scenes.LD27Scene;
	import com.zehfernando.ld27.display.scenes.game.GameScene;
	import com.zehfernando.utils.AppUtils;

	import flash.filters.DropShadowFilter;
	/**
	 * @author zeh fernando
	 */
	public class EndScene extends LD27Scene {

		// Constants
		private static const ACTION_ANY_KEY:String = "anyKey";

		// Instances
		private var textInstructions:TextSprite;
		private var binder:KeyActionBinder;


		// ================================================================================================================
		// CONSTRUCTOR ----------------------------------------------------------------------------------------------------

		public function EndScene(__userWon:Boolean) {
			super();

			var text:String;

			if (__userWon) {
				text = "YOU DID IT!\n\nYOU SCRAMBLE AWAY FROM THE DIRTY PLANET,\nASKING YOURSELF WHY WERE YOU THERE ANYWAY.\nYOUR FUEL SCREAMS, AND YOU REMEMBER:\nFUEL IS CHEAPER IN NEW JERSEY.\n\n(PRESS ANY KEY)";
			} else {
				text = "USE SPENT ALL YOUR FUEL.\n\nAS YOU WAIT FOR THE INEVITABLE HUMAN MISSION\nTHAT WILL COME AND DISSECT YOU,\nYOU CANNOT HELP BUT REMIND YOURSELF THAT\nYOU NEVER SAW PARIS.\n\n(PRESS ANY KEY)";
			}

			textInstructions = new TextSprite(FontLibrary.KUBUS_BOLD, 50, 0xffffff, 1);
			textInstructions.text = text;
			textInstructions.blockAlignHorizontal = TextSpriteAlign.CENTER;
			textInstructions.tracking = 2;
			textInstructions.blockAlignVertical = TextSpriteAlign.MIDDLE;
			textInstructions.align = TextSpriteAlign.CENTER;
			textInstructions.filters = [new DropShadowFilter(1, 45, 0x000000, 1, 2, 2)];
			addChild(textInstructions);

			binder = new KeyActionBinder(AppUtils.getStage());
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.BUTTON_ACTION_DOWN);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.BUTTON_ACTION_LEFT);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.BUTTON_ACTION_RIGHT);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.BUTTON_ACTION_UP);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.DPAD_DOWN);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.DPAD_LEFT);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.DPAD_RIGHT);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.DPAD_UP);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.XBOX_START);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.XBOX_BUTTON_ACTION_A);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.XBOX_BUTTON_ACTION_B);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.XBOX_BUTTON_ACTION_X);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.XBOX_BUTTON_ACTION_Y);
			binder.addKeyboardActionBinding(ACTION_ANY_KEY);
			binder.onActionActivated.add(onAnyKeyPressed);
		}

		// ================================================================================================================
		// INTERNAL INTERFACE ---------------------------------------------------------------------------------------------

		override protected function redrawWidth():void {
			super.redrawWidth();
			textInstructions.x = _width * 0.5;
		}

		override protected function redrawHeight():void {
			super.redrawHeight();
			textInstructions.y = _height * 0.5;
		}

		private function showNextScene():void {
			getSceneViewer().showScene(new GameScene());
		}

		private function onAnyKeyPressed(__action:String):void {
			binder.onActionActivated.removeAll();
			binder.stop();
			showNextScene();
		}


		// ================================================================================================================
		// PUBLIC INTERFACE -----------------------------------------------------------------------------------------------

		override public function dispose():void {
			binder.stop();
			binder = null;
			super.dispose();
		}

	}
}
