package com.zehfernando.ld27.display.scenes.game {
	import com.zehfernando.ld27.application.LD27;
	import com.zehfernando.ld27.display.scenes.LD27Scene;
	import com.zehfernando.ld27.display.scenes.end.EndScene;
	import com.zehfernando.ld27.display.scenes.game.world.WorldHUD;
	import com.zehfernando.ld27.display.scenes.game.world.WorldModel;
	import com.zehfernando.ld27.display.scenes.game.world.WorldView;
	import com.zehfernando.models.GameLooper;
	import com.zehfernando.net.assets.AssetLibrary;

	import flash.events.Event;
	/**
	 * @author zeh fernando
	 */
	public class GameScene extends LD27Scene {

		// Instances
		private var looper:GameLooper;

		private var worldModel:WorldModel;
		private var worldView:WorldView;
		private var worldHUD:WorldHUD;

		private var isRunning:Boolean;


		// ================================================================================================================
		// CONSTRUCTOR ----------------------------------------------------------------------------------------------------

		public function GameScene() {
			super();

			isRunning = false;

			// Create view
			// Have to wait until it's ready because of starling's delayed initialization
			worldView = new WorldView();
			worldView.addEventListener(WorldView.EVENT_READY, onViewReady);
			addChild(worldView);

			addEventListener(EVENT_STARTED_SHOWING, onStartedShowing);
		}

		// ================================================================================================================
		// INTERNAL INTERFACE ---------------------------------------------------------------------------------------------

		override protected function redrawWidth():void {
			super.redrawWidth();
			worldView.width = _width;
			if (isRunning) worldHUD.width = _width;
		}

		override protected function redrawHeight():void {
			super.redrawHeight();
			worldView.height = _height;
			if (isRunning) worldHUD.y = height - worldHUD.height;
		}


		// ================================================================================================================
		// EVENT INTERFACE ------------------------------------------------------------------------------------------------

		private function onStartedShowing(__e:flash.events.Event):void {
			worldView.getStarling().root.visible = true;
		}

		private function onViewReady(__e:Event):void {
			// View is ready, so start everything else

			if (!isRunning) {
				isRunning = true;

				// Looper
				//looper = new GameLooper();
				looper = new GameLooper(false, LD27.TARGET_FPS);
				looper.timeScale = 1;
				looper.onTicked.add(onTick);
				looper.onTickedOncePerVisualFrame.add(onTickVisual);

				// Interface on top
				worldHUD = new WorldHUD();
				addChild(worldHUD);

				// Actual world
				worldModel = new WorldModel(AssetLibrary.getLibrary().getJSON(LD27.ASSET_NAME_JSON_GAME)["levels"][0], worldView, worldHUD);

				redrawWidth();
				redrawHeight();
			}
		}

		private function onTick(__currentTimeSeconds:Number, __tickDeltaTimeSeconds:Number, __currentTick:int):void {
			// Go through body list and update sprite positions/rotations
			worldModel.update(__currentTimeSeconds, __tickDeltaTimeSeconds);

			if (worldModel.playerWon) {
				looper.pause();
				getSceneViewer().showScene(new EndScene(true));
			}
			if (worldModel.playerLost) {
				looper.pause();
				getSceneViewer().showScene(new EndScene(false));
			}
		}

		private function onTickVisual(__currentTimeSeconds:Number, __tickDeltaTimeSeconds:Number, __currentTick:int):void {
			worldModel.draw(__currentTimeSeconds, __tickDeltaTimeSeconds);
		}

		// ================================================================================================================
		// PUBLIC INTERFACE -----------------------------------------------------------------------------------------------

		override public function dispose():void {
			looper.pause();
			looper.dispose();
			looper = null;

			worldModel.dispose();
			worldModel = null;

			removeChild(worldView);
			worldView.dispose();
			worldView = null;

			removeEventListener(EVENT_STARTED_SHOWING, onStartedShowing);
			super.dispose();
		}

	}
}
