package com.zehfernando.ld27.display.scenes.game.actors {
	import nape.phys.Body;
	import nape.phys.BodyType;
	import nape.shape.Polygon;

	import com.zehfernando.ld27.application.LD27;
	import com.zehfernando.ld27.display.scenes.game.world.WorldModel;
	import com.zehfernando.ld27.starling.templates.ShapeSprite;

	import flash.geom.ColorTransform;
	/**
	 * @author zeh fernando
	 */
	public class Building extends Actor {

		// Assets
		[Embed(source="/../embed/sprites.swf", symbol="Building")]
		public static const SPRITE_BUILDING:Class;

		// Constants
		public static const ASSET_WIDTH:Number = 206;
		public static const ASSET_HEIGHT:Number = 946;
		public static const FLOOR_HEIGHT:Number = 57;

		// Properties
		private var _x:Number;
		private var y:Number;			// Base (ground)
		private var _height:Number;
		private var floors:int;

		// Instances
		private var body:Body;
		private var sprite:ShapeSprite;

		// ================================================================================================================
		// CONSTRUCTOR ----------------------------------------------------------------------------------------------------

		public function Building(__x:Number, __y:Number, __floors:int) {
			super();

			_x = __x;
			y = __y;
			floors = __floors;
		}


		// ================================================================================================================
		// PUBLIC INTERFACE -----------------------------------------------------------------------------------------------
		override public function start():void {
			super.start();

			_height = FLOOR_HEIGHT * floors + 15;

			//var colorMult:Number = 1-Math.random() * 0.3;
			//var cf:ColorTransform = new ColorTransform(colorMult, colorMult, colorMult, 1);

			// Create sprite
			sprite = new ShapeSprite(SPRITE_BUILDING, 1, 1, null, [LD27.FILTER_DROP_SHADOW], LD27.FILTER_DROP_SHADOW.blurX);
			sprite.x = _x;
			sprite.y = y - _height;
			sprite.width = ASSET_WIDTH;
			sprite.height = ASSET_HEIGHT;
			sprite.flatten();
			worldModel.getWorldView().getBackgroundContainer().addChild(sprite);

			// Create body
			body = new Body(BodyType.STATIC);
			body.position.setxy(x, y - _height);
			body.shapes.add(new Polygon(Polygon.rect(0, 0, ASSET_WIDTH, _height)));
			body.shapes.at(0).filter.collisionGroup = WorldModel.COLLISION_GROUP_LANDABLE;
			body.userData["name"] = "building";
			//body.shapes.at(0).filter.collisionMask = ~(WorldModel.COLLISION_GROUP_PLAYER | WorldModel.COLLISION_GROUP_PARTICLES | WorldModel.COLLISION_GROUP_HUMANS); // Collides with everything, except the player and particles and humans
			body.debugDraw = LD27.DEBUG_DRAW_ALL_DEBUG_BODIES;
			worldModel.getSpace().bodies.add(body);
		}

		override public function stop():void {
			super.stop();

			worldModel.getSpace().bodies.remove(body);
			body = null;

			worldModel.getWorldView().getBackgroundContainer().removeChild(sprite);
			sprite.dispose();
			sprite = null;
		}


		// ================================================================================================================
		// ACCESSOR INTERFACE ---------------------------------------------------------------------------------------------

		public function get x():Number {
			return _x;
		}

		public function get height():Number {
			return _height;
		}
	}
}
