package com.zehfernando.ld27.display.scenes.game.actors {
	import com.zehfernando.data.types.Color;
	import com.zehfernando.ld27.application.LD27;
	import com.zehfernando.ld27.starling.templates.ShapeSprite;
	import com.zehfernando.utils.MathUtils;
	import com.zehfernando.utils.RandomGenerator;

	import flash.geom.ColorTransform;
	/**
	 * @author zeh fernando
	 */
	public class Cloud extends Actor {

		// Assets
		[Embed(source="/../embed/sprites.swf", symbol="Cloud1")]
		public static const SPRITE_CLOUD_1:Class;

		[Embed(source="/../embed/sprites.swf", symbol="Cloud2")]
		public static const SPRITE_CLOUD_2:Class;

		[Embed(source="/../embed/sprites.swf", symbol="Cloud3")]
		public static const SPRITE_CLOUD_3:Class;

		[Embed(source="/../embed/sprites.swf", symbol="Cloud4")]
		public static const SPRITE_CLOUD_4:Class;

		[Embed(source="/../embed/sprites.swf", symbol="Cloud5")]
		public static const SPRITE_CLOUD_5:Class;

		[Embed(source="/../embed/sprites.swf", symbol="Cloud6")]
		public static const SPRITE_CLOUD_6:Class;

		// Properties
		private var x:Number;
		private var y:Number;
		private var alpha:Number;
		private var velocity:Number;

		// Instances
		private var sprite:ShapeSprite;


		// ================================================================================================================
		// CONSTRUCTOR ----------------------------------------------------------------------------------------------------

		public function Cloud(__x:Number, __y:Number, __alpha:Number, __velocity:Number) {
			super();

			x = __x;
			y = __y;
			alpha = __alpha;
			velocity = __velocity;
		}


		// ================================================================================================================
		// PUBLIC INTERFACE -----------------------------------------------------------------------------------------------

		override public function start():void {

			// Sprite
			var spriteClass:Class = RandomGenerator.getFromArray([SPRITE_CLOUD_1, SPRITE_CLOUD_2, SPRITE_CLOUD_3, SPRITE_CLOUD_4, SPRITE_CLOUD_5, SPRITE_CLOUD_6]);

			var ct:ColorTransform = Color.fromRRGGBB(Color.interpolateRRGGBB(0xffffff, 0x7ebdff, alpha)).toColorTransformMultiplied();

			sprite = new ShapeSprite(spriteClass, 1, 1.5, ct, [LD27.FILTER_DROP_SHADOW], LD27.FILTER_DROP_SHADOW.blurX + 1);
			sprite.x = x;
			sprite.y = y;
			sprite.scaleX = sprite.scaleY = MathUtils.map(alpha, 0, 1, 0, 1) + RandomGenerator.getInRange(0, 0.1);
			sprite.flatten();
			worldModel.getWorldView().getParticlesContainer().addChild(sprite);
		}

		override public function stop():void {
			super.stop();

			worldModel.getWorldView().getParticlesContainer().removeChild(sprite);
			sprite.dispose();
			sprite = null;
		}

		override public function update(__currentTimeSeconds:Number, __deltaTimeSeconds:Number):void {
			// Called 60 times per second
//			if (isNaN(timeCreated)) timeCreated = __currentTimeSeconds;
//			if (__currentTimeSeconds > timeCreated + timeToLive) {
//				suicide();
//			}
		}

		override public function draw(__currentTimeSeconds:Number, __deltaTimeSeconds:Number):void {
			// Called once per visual frame
			sprite.x += __deltaTimeSeconds * velocity;

			var halfWidth:Number = sprite.width * 0.5 * sprite.scaleX * 2; // hardcoded quality

			// If it goes out of the world, should come back
			if (sprite.x > worldModel.getWorldWidth() + halfWidth) {
				sprite.x = -halfWidth;
			} else if (sprite.x < -halfWidth) {
				sprite.x = worldModel.getWorldWidth() + halfWidth;
			}
		}
	}
}
