package com.zehfernando.ld27.display.scenes.game.actors {
	import nape.phys.Body;
	import nape.phys.BodyType;
	import nape.shape.Polygon;

	import com.zehfernando.ld27.application.LD27;
	import com.zehfernando.ld27.display.scenes.game.world.WorldModel;
	import com.zehfernando.ld27.starling.templates.ShapeSprite;
	import com.zehfernando.utils.console.error;
	/**
	 * @author zeh fernando
	 */
	public class Ground extends Actor {

		// Assets
		[Embed(source="/../embed/sprites.swf", symbol="GroundGrass")]
		public static const SPRITE_GROUND_GRASS:Class;

		// Properties
		private var type:String;
		private var width:Number;
		private var height:Number;

		// Instances
		private var body:Body;
		private var sprite:ShapeSprite;

		// ================================================================================================================
		// CONSTRUCTOR ----------------------------------------------------------------------------------------------------

		public function Ground(__type:String, __width:Number, __height:Number) {
			super();

			type = __type;
			width = __width;
			height = __height;
		}


		// ================================================================================================================
		// PUBLIC INTERFACE -----------------------------------------------------------------------------------------------
		override public function start():void {
			super.start();

			var _class:Class = null;

			switch (type) {
				case "grass":
					_class = SPRITE_GROUND_GRASS;
					break;
			}

			if (_class == null) {
				error("Could not create a ground of type ["+type+"]!!!");
				return;
			}

			// Create sprite
			sprite = new ShapeSprite(_class);
			sprite.y = worldModel.getWorldHeight() - height;
			sprite.width = width;
			sprite.height = height;
			sprite.flatten();
			worldModel.getWorldView().getBackgroundContainer().addChild(sprite);

			// Create body
			body = new Body(BodyType.STATIC);
			body.position.setxy(0, worldModel.getWorldHeight() - height);
			body.shapes.add(new Polygon(Polygon.rect(0, 0, width, height)));
			body.shapes.at(0).filter.collisionGroup = WorldModel.COLLISION_GROUP_LANDABLE;
			body.userData["name"] = "ground";
			//body.shapes.at(0).filter.collisionMask = ~(WorldModel.COLLISION_GROUP_PLAYER | WorldModel.COLLISION_GROUP_PARTICLES | WorldModel.COLLISION_GROUP_HUMANS); // Collides with everything, except the player and particles and humans
			body.debugDraw = LD27.DEBUG_DRAW_ALL_DEBUG_BODIES;
			worldModel.getSpace().bodies.add(body);
		}

		override public function stop():void {
			super.stop();

			worldModel.getSpace().bodies.remove(body);
			body = null;

			worldModel.getWorldView().getBackgroundContainer().removeChild(sprite);
			sprite.dispose();
			sprite = null;
		}
	}
}
