package com.zehfernando.ld27.display.scenes.game.actors {
	import nape.dynamics.CollisionArbiter;
	import nape.geom.Vec2;
	import nape.phys.Body;
	import nape.phys.BodyType;
	import nape.shape.Polygon;

	import com.zehfernando.ld27.application.LD27;
	import com.zehfernando.ld27.display.scenes.game.world.WorldModel;
	import com.zehfernando.ld27.starling.templates.MovieClipSprite;
	import com.zehfernando.utils.console.log;
	/**
	 * @author zeh fernando
	 */
	public class Human extends Actor {

		// Assets
		[Embed(source="/../embed/sprites.swf", symbol="HumanWalk")]
		public static const SPRITE_HUMAN_WALK:Class;

		// Constants
		public static const VELOCITY_WALK:Number = 30;			// Walking speed, in pixels/second
		public static const MAXIMUM_BEAM_SPEED:Number = 90;		// Maximum vertical attraction speed
		public static const BEAM_CAPTURE_DISTANCE:Number = 30;	// How close this human has to be to the attractor beam to be considered captured

		public static const WIDTH:Number = 15;					// Bounding box dimensions
		public static const HEIGHT:Number = 28;
		public static const ANIMATION_FRAMERATE:Number = 15;	// Framerate per second of the movieclip animation

		// Properties
		private var x:Number;
		private var y:Number;
		private var velocity:Number;
		private var frameOffset:int;
		private var attractX:Number;							// Additional attraction velocity
		private var attractY:Number;

		private var isTouchingTheGround:Boolean;

		private var i:int;										// For speed

		private var lastKnownGoodPosition:Vec2;

		// Instances
		private var body:Body;
		private var sprite:MovieClipSprite;
		private var attractBody:Body;
		private var attractVec:Vec2;


		// ================================================================================================================
		// CONSTRUCTOR ----------------------------------------------------------------------------------------------------

		public function Human(__x:Number, __y:Number, __velocity:Number) {
			super();

			x = __x;
			y = __y;
			velocity = __velocity;
		}


		// ================================================================================================================
		// EVENT INTERFACE ------------------------------------------------------------------------------------------------

		private function onAttractionStart(__body:Body):void {
			attractBody = __body;
		}

		private function onAttractionEnd(__body:Body):void {
			attractBody = null;
		}

		// ================================================================================================================
		// PUBLIC INTERFACE -----------------------------------------------------------------------------------------------

		override public function start():void {

			// Properties
			attractBody = null;
			isTouchingTheGround = true;
			lastKnownGoodPosition = new Vec2(0, 0);

			// Create body
			body = new Body(BodyType.DYNAMIC);
			body.shapes.add(new Polygon(Polygon.rect(-WIDTH/2, -HEIGHT, WIDTH, HEIGHT)));
			body.shapes.at(0).material.density = 0.1;
			body.shapes.at(0).material.elasticity = 0.1;
			body.shapes.at(0).filter.collisionGroup = WorldModel.COLLISION_GROUP_HUMANS;
			body.shapes.at(0).filter.collisionMask = ~(WorldModel.COLLISION_GROUP_PLAYER | WorldModel.COLLISION_GROUP_PARTICLES | WorldModel.COLLISION_GROUP_HUMANS); // Collides with everything, except the player and particles and humans
			body.userData["name"] = "human";
			body.allowRotation = false;
			body.position.setxy(x, y);
			body.velocity.setxy(velocity, 0);
			body.cbTypes.add(WorldModel.CB_TYPE_BEAMABLE);
			body.debugDraw = LD27.DEBUG_DRAW_ALL_DEBUG_BODIES;
			body.userData.onAttractionStart = onAttractionStart;
			body.userData.onAttractionEnd = onAttractionEnd;
			worldModel.getSpace().bodies.add(body);

			// Body sprite
			sprite = new MovieClipSprite(SPRITE_HUMAN_WALK, 1, 1, null, [LD27.FILTER_DROP_SHADOW], LD27.FILTER_DROP_SHADOW.blurX + 1);
//			sprite.rotation = rotationStart;
//			sprite.alpha = alphaStart;
//			sprite.scaleX = sprite.scaleY = scaleStart;
			worldModel.getWorldView().getParticlesContainer().addChild(sprite);

			frameOffset = Math.floor(Math.random() * sprite.totalFrames);
		}

		override public function stop():void {
			super.stop();

			worldModel.getSpace().bodies.remove(body);
			body = null;

			worldModel.getWorldView().getParticlesContainer().removeChild(sprite);
			sprite.dispose();
			sprite = null;
		}

		override public function update(__currentTimeSeconds:Number, __deltaTimeSeconds:Number):void {
			// Called 60 times per second
			if (Math.abs(body.velocity.x) < 2 && attractBody == null && isTouchingTheGround) {
				// Hitting a wall: walk in the other directions
				body.velocity.x *= -1;
				velocity *= -1;
			}

			if (attractBody != null) {
				// Being attracted to a body, influence the velocity
				attractVec = attractBody.position.sub(body.position);
				attractX = attractVec.x;
				attractY = attractVec.y;

				lastKnownGoodPosition.y = 0;

				//log("distr = " + Vec2.distance(attractBody.position, body.position));
				if (Vec2.distance(attractBody.position, body.position) < BEAM_CAPTURE_DISTANCE) {
					worldModel.getPlayerActor().addCapturedTarget();
					suicide();
				}
			} else {
				// Not being attracted to a body
				attractX = 0;
				attractY = 0;
			}

			body.applyImpulse(new Vec2((velocity+attractX)/60, attractY));
			body.velocity.x = velocity < 0 ? Math.max(velocity, body.velocity.x) : Math.min(velocity, body.velocity.x);
			body.velocity.y = body.velocity.y < 0 ? Math.max(-MAXIMUM_BEAM_SPEED, body.velocity.y) : body.velocity.y;
			//body.velocity.setxy(velocity + attractX, attractY);

			isTouchingTheGround = false;
			if (body.arbiters.length > 0) {
				for (i = 0; i < body.arbiters.length; i++) {
					if (body.arbiters.at(i) is CollisionArbiter) {
						isTouchingTheGround = true;
						break;
					}
				}
			}

			if (!isTouchingTheGround) {
				// Going to fall! Avoid falling by turning
				if (lastKnownGoodPosition.y > 0 && attractBody == null) {
					velocity *= -1;
					body.position.setxy(lastKnownGoodPosition.x + velocity * 0.2, lastKnownGoodPosition.y);
					body.velocity.setxy(velocity, 0);
				}
			} else {
				// Still touching ground
				lastKnownGoodPosition.setxy(body.position.x, body.position.y);
			}
		}

		override public function draw(__currentTimeSeconds:Number, __deltaTimeSeconds:Number):void {
			// Called once per visual frame
			sprite.x = body.position.x;
			sprite.y = body.position.y;
			sprite.scaleX = velocity > 0 ? 1 : -1;
			sprite.setFrame((Math.floor(Human.ANIMATION_FRAMERATE * __currentTimeSeconds) + frameOffset) % sprite.totalFrames);
		}
	}
}
