package com.zehfernando.ld27.display.scenes.game.actors {
	import nape.callbacks.CbEvent;
	import nape.callbacks.InteractionCallback;
	import nape.callbacks.InteractionListener;
	import nape.callbacks.InteractionType;
	import nape.constraint.WeldJoint;
	import nape.dynamics.InteractionFilter;
	import nape.geom.Ray;
	import nape.geom.RayResult;
	import nape.geom.Vec2;
	import nape.phys.Body;
	import nape.phys.BodyType;
	import nape.shape.Circle;
	import nape.shape.Polygon;
	import com.zehfernando.audio.SoundPlayer;
	import com.zehfernando.input.binding.GamepadControls;
	import com.zehfernando.ld27.application.LD27;
	import com.zehfernando.ld27.assets.SoundLibrary;
	import com.zehfernando.ld27.display.scenes.game.actors.particles.ThrustParticle;
	import com.zehfernando.ld27.display.scenes.game.world.WorldModel;
	import com.zehfernando.ld27.starling.templates.ShapeSprite;
	import com.zehfernando.transitions.Equations;
	import com.zehfernando.utils.AppUtils;
	import com.zehfernando.utils.MathUtils;
	import com.zehfernando.utils.RandomGenerator;
	import flash.filters.BlurFilter;
	import flash.ui.Keyboard;
	/**
	 * @author zeh fernando
	 */
	public class Player extends Actor {

		// Assets
		[Embed(source="/../embed/sprites.swf", symbol="Ship")]
		public static const SPRITE_SHIP:Class;

		[Embed(source="/../embed/sprites.swf", symbol="ShipLandingUp")]
		public static const SPRITE_SHIP_LANDING_UP:Class;

		[Embed(source="/../embed/sprites.swf", symbol="ShipLandingDown")]
		public static const SPRITE_SHIP_LANDING_DOWN:Class;

		[Embed(source="/../embed/sprites.swf", symbol="ShipTractorBeam")]
		public static const SPRITE_SHIP_TRACTOR_BEAM:Class;

		// Constants
		public static const MAX_FUEL:Number = 10;										// Max fuel, in seconds of use

		private static const ACTION_THRUSTER:String = "thruster";
		private static const ACTION_ROTATE_AXIS:String = "rotate-axis";
		private static const ACTION_ROTATE_LEFT_DIGITAL:String = "rotate-left-digital";
		private static const ACTION_ROTATE_RIGHT_DIGITAL:String = "rotate-right-digital";

		private static const MAX_THRUSTER_PARTICLES_PER_SECOND:Number = 20;				// Each one counts for all 4 thrusters //240;

		private static const THRUSTER_INTERNAL_POS:Vec2 = new Vec2(27, 15);				// Position for internal thruster
		private static const THRUSTER_EXTERNAL_POS:Vec2 = new Vec2(52, 10);				// Position for external thruster
		private static const THRUSTER_FORCE:Number = 20;								// Impulse applied when thruster is on (in pixels/second?)
		private static const MAX_THRUSTER_SOUNDS_PER_SECOND:Number = 12;				// How many times the thruster sound can be played per second
		private static const MAX_ROTATION:Number = 40;									// In degrees

		private static const TRACTOR_BEAM_HEIGHT:Number = 100;							// How close it has to be to the ground before it extends the landing gears

		private static const LANDING_GEAR_SENSOR_DISTANCE:Number = 300;					// How close it has to be to the ground before it extends the landing gears
		private static const LANDING_GEAR_ROTATION_RETRACTED:Number = Math.PI * 0.45;	// Angle of right landing gear
		private static const LANDING_GEAR_ROTATION_EXTENDED:Number = -0.2;
		private static const LANDING_GEAR_POSITION_RETRACTED:Number = 10;				// Vertical position inside the landing gear
		private static const LANDING_GEAR_POSITION_EXTENDED:Number = 0;

		private static const TIME_LANDING_GEAR_CONTRACT_EXTEND:Number = 0.8;			// Time in seconds for the landing gear to contract or extend completely
		private static const TIME_TRACTOR_BEAM_POWERS:Number = 0.5;						// Time in seconds for the tractor beam to fully power

		// Properties
		private var _fuel:Number;
		private var lastTimeThrusterParticleCreated:Number;								// In seconds
		private var lastTimeThrusterSoundPlayed:Number;									// In seconds
		private var cameraX:Number;
		private var cameraY:Number;

		private var startX:Number;
		private var startY:Number;

		private var numTargetsInBeam:int;
		private var _numCapturedTargets:int;

		private var tractorBeamPower:Number;					// 0 (off) to 1 (on)
		private var landingGearExtended:Number;					// 0 (retracted) to 1 (extended)
		private var landingGearExtendedEased:Number;			// 0 (retracted) to 1 (extended) - for less garbage collection
		private var _isOutOfFuel:Boolean;

		// Instances for speed
		private var particleVelocity:Vec2;
		private var thrustVel:Vec2;
		private var desiredRotation:Number;
		private var axis:Number;

//		private var lastImpactNormal:Vec2;						// Angle to jump based on last impact
//		private var isTouchingASurface:Boolean;					// Is the player touching a surface now?
//		private var isTouchingASurfaceTolerant:Boolean;			// Is the player touching a surface or was he touching it just now?
//		private var playerVelocity:Number;						// Angular velocity, as controlled by the player
//		private var timeTouchedSurface:Number;					// Time in game seconds for the last time the surface was touched
//		private var timeLeftSurface:Number;						// Time in game seconds for the last time the surface was left
//		private var isInAJump:Boolean;							// Has jumped or not

		// Instances
		private var body:Body;
//		private var sprite:Sprite;
//		private var spriteBitmap:Bitmap;
		private var sprite:ShapeSprite;

		private var landingRay:Ray;
		private var landingFilter:InteractionFilter;

		private var thrusterExternalLeft:Vec2;
		private var thrusterInternalLeft:Vec2;
		private var thrusterInternalRight:Vec2;
		private var thrusterExternalRight:Vec2;

		private var landingRayOrigin:Vec2;
		private var landingRayDirection:Vec2;

		private var landingGearLeft:Body;
		private var landingGearRight:Body;

		private var spriteLandingGearLeftUp:ShapeSprite;
		private var spriteLandingGearLeftDown:ShapeSprite;
		private var spriteLandingGearRightUp:ShapeSprite;
		private var spriteLandingGearRightDown:ShapeSprite;

		private var landingGearLeftJoint:WeldJoint;
		private var landingGearRightJoint:WeldJoint;

		private var spriteTractorBeam:ShapeSprite;

		// ================================================================================================================
		// CONSTRUCTOR ----------------------------------------------------------------------------------------------------

		public function Player(__startX:Number, __startY:Number) {
			super();

			// Don't do anything (other than setting parameters) here
			startX = __startX;
			startY = __startY;
		}


		// ================================================================================================================
		// INTERNAL INTERFACE ---------------------------------------------------------------------------------------------

		override public function start():void {
			// Properties

			_fuel = MAX_FUEL;
			_isOutOfFuel = false;
			numTargetsInBeam = 0;
			_numCapturedTargets = 0;

			particleVelocity = new Vec2(0, 0);
			thrustVel = new Vec2(0, 0);
			desiredRotation = 0;
			axis = 0;
			landingGearExtended = 0;
			tractorBeamPower = 0;

			landingRay = new Ray(Vec2.weak(), Vec2.weak());
			landingRay.maxDistance = LANDING_GEAR_SENSOR_DISTANCE;
			landingFilter = new InteractionFilter(WorldModel.COLLISION_GROUP_LANDABLE, ~(WorldModel.COLLISION_GROUP_PARTICLES|WorldModel.COLLISION_GROUP_PLAYER|WorldModel.COLLISION_GROUP_HUMANS));
			landingRayOrigin = new Vec2(0, 0);
			landingRayDirection = new Vec2(0, 1);

			thrusterExternalLeft = new Vec2(-THRUSTER_EXTERNAL_POS.x, THRUSTER_EXTERNAL_POS.y);
			thrusterInternalLeft = new Vec2(-THRUSTER_INTERNAL_POS.x, THRUSTER_INTERNAL_POS.y);
			thrusterInternalRight = new Vec2(THRUSTER_INTERNAL_POS.x, THRUSTER_INTERNAL_POS.y);
			thrusterExternalRight = new Vec2(THRUSTER_EXTERNAL_POS.x, THRUSTER_EXTERNAL_POS.y);

			// Key bindings
			if (AppUtils.isAndroid()) {
				// Android and OUYA: use native controller
				worldModel.getKeyBinder().addGamepadActionBinding(ACTION_THRUSTER, GamepadControls.BUTTON_ACTION_DOWN);
				worldModel.getKeyBinder().addGamepadSensitiveActionBinding(ACTION_ROTATE_AXIS, GamepadControls.STICK_LEFT_X, -1, -1, 1);
			} else {
				// PC: use keyboard and XBox 360 controller
				worldModel.getKeyBinder().addKeyboardActionBinding(ACTION_THRUSTER, Keyboard.SPACE);
				worldModel.getKeyBinder().addKeyboardActionBinding(ACTION_ROTATE_LEFT_DIGITAL, Keyboard.LEFT);
				worldModel.getKeyBinder().addKeyboardActionBinding(ACTION_ROTATE_RIGHT_DIGITAL, Keyboard.RIGHT);
				worldModel.getKeyBinder().addKeyboardActionBinding(ACTION_ROTATE_LEFT_DIGITAL, Keyboard.A);
				worldModel.getKeyBinder().addKeyboardActionBinding(ACTION_ROTATE_RIGHT_DIGITAL, Keyboard.D);

				worldModel.getKeyBinder().addGamepadActionBinding(ACTION_THRUSTER, GamepadControls.XBOX_BUTTON_ACTION_A);
				worldModel.getKeyBinder().addGamepadSensitiveActionBinding(ACTION_ROTATE_AXIS, GamepadControls.STICK_LEFT_X, -1, -1, 1);
			}

			// Create body
			body = new Body(BodyType.DYNAMIC);

			// Creates all shapes - this is a bit hard-coded

			// Canopy
			body.shapes.add(new Circle(23, new Vec2(0, -4)));

			// Thrusters
			body.shapes.add(new Circle(10, new Vec2(-THRUSTER_EXTERNAL_POS.x, THRUSTER_EXTERNAL_POS.y)));
			body.shapes.add(new Circle(10, new Vec2(-THRUSTER_INTERNAL_POS.x, THRUSTER_INTERNAL_POS.y)));
			body.shapes.add(new Circle(10, new Vec2(THRUSTER_INTERNAL_POS.x, THRUSTER_INTERNAL_POS.y)));
			body.shapes.add(new Circle(10, new Vec2(THRUSTER_EXTERNAL_POS.x, THRUSTER_EXTERNAL_POS.y)));

			// Hull
			var vertices:Vector.<Vec2> = new Vector.<Vec2>();
			vertices.push(new Vec2(0, -10));
			vertices.push(new Vec2(40, -8));
			vertices.push(new Vec2(70, 0));
			vertices.push(new Vec2(70, 3));
			vertices.push(new Vec2(46, 16));
			vertices.push(new Vec2(0, 20));
			vertices.push(new Vec2(-46, 16));
			vertices.push(new Vec2(-70, 3));
			vertices.push(new Vec2(-70, 0));
			vertices.push(new Vec2(-40, -8));
			body.shapes.add(new Polygon(vertices));

			for (var i:int = 0; i < body.shapes.length; i++) {
				body.shapes.at(i).material.density = 1; // Default = 1. g/pixel/pixel
				body.shapes.at(i).material.elasticity = 0;
				body.shapes.at(i).material.rollingFriction = 0;
				body.shapes.at(i).material.dynamicFriction = 0.5;
				body.shapes.at(i).material.staticFriction = 1;
				body.shapes.at(i).filter.collisionGroup = WorldModel.COLLISION_GROUP_PLAYER;
				body.shapes.at(i).filter.collisionMask = ~(WorldModel.COLLISION_GROUP_PARTICLES|WorldModel.COLLISION_GROUP_PLAYER); // Collides with everything, except particles and the player
			}

			body.position.setxy(startX, startY);
			body.angularVel = 0;
			body.debugDraw = LD27.DEBUG_DRAW_ALL_DEBUG_BODIES;
			cameraX = body.position.x;
			cameraY = body.position.y;
			worldModel.getSpace().bodies.add(body);

			// Landing gears
			var landingGearPosition:Vec2 = new Vec2(-40, 14);

			// Left
			landingGearLeft = new Body(BodyType.DYNAMIC);
			vertices = new Vector.<Vec2>();
			vertices.push(new Vec2(3, 0));
			vertices.push(new Vec2(1, 36));
			vertices.push(new Vec2(-1, 36));
			vertices.push(new Vec2(-3, 0));
			landingGearLeft.shapes.add(new Polygon(vertices));
			landingGearLeft.shapes.at(0).filter.collisionGroup = WorldModel.COLLISION_GROUP_PLAYER;
			landingGearLeft.shapes.at(0).filter.collisionMask = ~(WorldModel.COLLISION_GROUP_PARTICLES|WorldModel.COLLISION_GROUP_PLAYER); // Collides with everything, except particles and the player
			landingGearLeft.position.set(body.localPointToWorld(landingGearPosition));
			landingGearLeft.rotation = -LANDING_GEAR_ROTATION_EXTENDED;
			landingGearLeft.debugDraw = LD27.DEBUG_DRAW_ALL_DEBUG_BODIES;
			worldModel.getSpace().bodies.add(landingGearLeft);

			landingGearLeftJoint = new WeldJoint(body, landingGearLeft, landingGearPosition, Vec2.weak(0, 0), landingGearLeft.rotation - body.rotation);
			landingGearLeftJoint.debugDraw = LD27.DEBUG_DRAW_ALL_DEBUG_BODIES;
			worldModel.getSpace().constraints.add(landingGearLeftJoint);

			// Right
			landingGearPosition.x = -landingGearPosition.x;
			landingGearRight = new Body(BodyType.DYNAMIC);
			vertices = new Vector.<Vec2>();
			vertices.push(new Vec2(3, 0));
			vertices.push(new Vec2(1, 36));
			vertices.push(new Vec2(-1, 36));
			vertices.push(new Vec2(-3, 0));
			landingGearRight.shapes.add(new Polygon(vertices));
			landingGearRight.shapes.at(0).filter.collisionGroup = WorldModel.COLLISION_GROUP_PLAYER;
			landingGearRight.shapes.at(0).filter.collisionMask = ~(WorldModel.COLLISION_GROUP_PARTICLES|WorldModel.COLLISION_GROUP_PLAYER); // Collides with everything, except particles and the player
			landingGearRight.position.set(body.localPointToWorld(landingGearPosition));
			landingGearRight.rotation = LANDING_GEAR_ROTATION_EXTENDED;
			landingGearRight.debugDraw = LD27.DEBUG_DRAW_ALL_DEBUG_BODIES;
			worldModel.getSpace().bodies.add(landingGearRight);

			landingGearRightJoint = new WeldJoint(body, landingGearRight, landingGearPosition, Vec2.weak(0, 0), landingGearRight.rotation - body.rotation);
			landingGearRightJoint.debugDraw = LD27.DEBUG_DRAW_ALL_DEBUG_BODIES;
			worldModel.getSpace().constraints.add(landingGearRightJoint);

			// Landing gear sprites (down)
			spriteLandingGearLeftDown = new ShapeSprite(SPRITE_SHIP_LANDING_DOWN, 1, 1, null, [LD27.FILTER_DROP_SHADOW], LD27.FILTER_DROP_SHADOW.blurX + 1);
			spriteLandingGearLeftDown.flatten();
			worldModel.getWorldView().getSpritesContainer().addChild(spriteLandingGearLeftDown);

			spriteLandingGearRightDown = new ShapeSprite(SPRITE_SHIP_LANDING_DOWN, 1, 1, null, [LD27.FILTER_DROP_SHADOW], LD27.FILTER_DROP_SHADOW.blurX + 1);
			spriteLandingGearRightDown.flatten();
			worldModel.getWorldView().getSpritesContainer().addChild(spriteLandingGearRightDown);

			// Tractor beam
			vertices = new Vector.<Vec2>();
			vertices.push(new Vec2(10, 0));
			vertices.push(new Vec2(30, TRACTOR_BEAM_HEIGHT));
			vertices.push(new Vec2(-30, TRACTOR_BEAM_HEIGHT));
			vertices.push(new Vec2(-10, 0));
			var tractorShape:Polygon = new Polygon(vertices);
			tractorShape.filter = new InteractionFilter(-1, 1, WorldModel.COLLISION_GROUP_BEAM, -1);
			tractorShape.material.density = 0;
			tractorShape.sensorEnabled = true;
			body.shapes.add(tractorShape);
			body.cbTypes.add(WorldModel.CB_TYPE_BEAM);

			worldModel.getSpace().listeners.add(new InteractionListener(
				CbEvent.BEGIN, InteractionType.SENSOR,
				WorldModel.CB_TYPE_BEAM,
				WorldModel.CB_TYPE_BEAMABLE,
				onTractorBeamTouchStart
			));
			worldModel.getSpace().listeners.add(new InteractionListener(
				CbEvent.END, InteractionType.SENSOR,
				WorldModel.CB_TYPE_BEAM,
				WorldModel.CB_TYPE_BEAMABLE,
				onTractorBeamTouchEnd
			));

			// Tractor beam
			spriteTractorBeam = new ShapeSprite(SPRITE_SHIP_TRACTOR_BEAM, 1, 1, null, [new BlurFilter(2, 2, 2)], 4 + 1);
			spriteTractorBeam.flatten();
			worldModel.getWorldView().getSpritesContainer().addChild(spriteTractorBeam);

			// Body sprite
			sprite = new ShapeSprite(SPRITE_SHIP, 1, 1, null, [LD27.FILTER_DROP_SHADOW], LD27.FILTER_DROP_SHADOW.blurX + 1);
			sprite.flatten();
			worldModel.getWorldView().getSpritesContainer().addChild(sprite);

			// Landing gear sprites (up)
			spriteLandingGearLeftUp = new ShapeSprite(SPRITE_SHIP_LANDING_UP);
			spriteLandingGearLeftUp.flatten();
			worldModel.getWorldView().getSpritesContainer().addChild(spriteLandingGearLeftUp);

			spriteLandingGearRightUp = new ShapeSprite(SPRITE_SHIP_LANDING_UP);
			spriteLandingGearRightUp.flatten();
			worldModel.getWorldView().getSpritesContainer().addChild(spriteLandingGearRightUp);
		}

		private function onTractorBeamTouchStart(__cb:InteractionCallback):void {
			// A human has entered the tractor beam area
			if (numTargetsInBeam == 0) SoundPlayer.playSound(SoundLibrary.ATTRACT);

			numTargetsInBeam++;

			if (Boolean(__cb.int2.castBody.userData["onAttractionStart"])) {
				__cb.int2.castBody.userData["onAttractionStart"](body);
			}

			//log("sensor start => " + __cb.int2.castBody.shapes.at(0).userData.name);
		}

		private function onTractorBeamTouchEnd(__cb:InteractionCallback):void {
			// A human has left the tractor beam area
			numTargetsInBeam--;
			if (Boolean(__cb.int2.castBody.userData["onAttractionEnd"])) {
				__cb.int2.castBody.userData["onAttractionEnd"](body);
			}

			//log("sensor end " + __cb.int2.castBody.shapes.at(0).userData.name);
		}

		override public function stop():void {
			super.stop();

			worldModel.getSpace().constraints.remove(landingGearLeftJoint);
			landingGearLeftJoint = null;
			worldModel.getSpace().constraints.remove(landingGearRightJoint);
			landingGearRightJoint = null;

			worldModel.getSpace().bodies.remove(landingGearLeft);
			landingGearLeft = null;
			worldModel.getSpace().bodies.remove(landingGearRight);
			landingGearRight = null;

			worldModel.getSpace().bodies.remove(body);
			body = null;

			worldModel.getWorldView().getSpritesContainer().removeChild(spriteTractorBeam);
			spriteTractorBeam.dispose();
			spriteTractorBeam = null;

			worldModel.getWorldView().getSpritesContainer().removeChild(spriteLandingGearLeftUp);
			spriteLandingGearLeftUp.dispose();
			spriteLandingGearLeftUp = null;
			worldModel.getWorldView().getSpritesContainer().removeChild(spriteLandingGearRightUp);
			spriteLandingGearRightUp.dispose();
			spriteLandingGearRightUp = null;
			worldModel.getWorldView().getSpritesContainer().removeChild(spriteLandingGearLeftDown);
			spriteLandingGearLeftDown.dispose();
			spriteLandingGearLeftDown = null;
			worldModel.getWorldView().getSpritesContainer().removeChild(spriteLandingGearRightDown);
			spriteLandingGearRightDown.dispose();
			spriteLandingGearRightDown = null;

			worldModel.getWorldView().getSpritesContainer().removeChild(sprite);
			sprite.dispose();
			sprite = null;
		}

		private function createThrustParticle(__velocity:Vec2, __bodyVector:Vec2):void {
			var thrusterPosition:Vec2;
			particleVelocity.x = -__velocity.x;
			particleVelocity.y = -__velocity.y;
			particleVelocity.rotate(RandomGenerator.getInRange(-0.1, 0.1));
			particleVelocity.length *= (RandomGenerator.getInRange(1, 5));
			particleVelocity.x += body.velocity.x;
			particleVelocity.y += body.velocity.y;

			thrusterPosition = body.localPointToWorld(__bodyVector, true);
			worldModel.addActor(new ThrustParticle(thrusterPosition.x, thrusterPosition.y, particleVelocity));
		}

		override public function update(__currentTimeSeconds:Number, __deltaTimeSeconds:Number):void {
			// Called 60 times per second

			if (!_isOutOfFuel) {
				// Read inputs

				// Ship thruster
				if (worldModel.getKeyBinder().isActionActivated(ACTION_THRUSTER)) {
					thrustVel.x = 0;
					thrustVel.y = -THRUSTER_FORCE;
					thrustVel.rotate(body.rotation);
					body.applyImpulse(thrustVel);

					// Remove fuel
					_fuel -= __deltaTimeSeconds;
					if (_fuel <= 0) {
						// Ran out of fuel
						_fuel = 0;
						if (!LD27.DEBUG_GAME_IGNORE_EMPTY_FUEL) _isOutOfFuel = true;
					}

					// Plays thruster sound
					if (isNaN(lastTimeThrusterSoundPlayed) || __currentTimeSeconds > lastTimeThrusterSoundPlayed + (1/MAX_THRUSTER_SOUNDS_PER_SECOND)) {
						SoundPlayer.playSound(SoundLibrary.ROCKET, 0, RandomGenerator.getInRange(0.2, 0.6), 0, Math.random() * 1);
						if (isNaN(lastTimeThrusterSoundPlayed)) {
							lastTimeThrusterSoundPlayed = __currentTimeSeconds;
						} else {
							lastTimeThrusterSoundPlayed += (1/MAX_THRUSTER_SOUNDS_PER_SECOND);
						}
					}

					// Create thrust particles
					if (isNaN(lastTimeThrusterParticleCreated) || __currentTimeSeconds > lastTimeThrusterParticleCreated + (1/MAX_THRUSTER_PARTICLES_PER_SECOND)) {
						createThrustParticle(thrustVel, thrusterExternalLeft);
						createThrustParticle(thrustVel, thrusterInternalLeft);
						createThrustParticle(thrustVel, thrusterInternalRight);
						createThrustParticle(thrustVel, thrusterExternalRight);
						if (isNaN(lastTimeThrusterParticleCreated)) {
							lastTimeThrusterParticleCreated = __currentTimeSeconds;
						} else {
							lastTimeThrusterParticleCreated += (1/MAX_THRUSTER_PARTICLES_PER_SECOND);
						}
					}
				} else {
					lastTimeThrusterSoundPlayed = NaN;
					lastTimeThrusterParticleCreated = NaN;
				}
			}

			// Tractor beam
			if (numTargetsInBeam > 0) {
				// At least one target in tractor beam range
				if (tractorBeamPower != 1) {
					// Need to turn tractor beam on
					tractorBeamPower = Math.min(tractorBeamPower + __deltaTimeSeconds / TIME_TRACTOR_BEAM_POWERS, 1);
				}
			} else {
				// No targets in beam range
				if (tractorBeamPower != 0) {
					// Need to contract landing gear
					tractorBeamPower = Math.max(tractorBeamPower - __deltaTimeSeconds / TIME_TRACTOR_BEAM_POWERS, 0);
				}
			}

			// Ship rotation
			desiredRotation = 0;

			// Find rotation based on stick
			if (!_isOutOfFuel) {
				axis = worldModel.getKeyBinder().getActionValue(ACTION_ROTATE_AXIS);
				if (worldModel.getKeyBinder().isActionActivated(ACTION_ROTATE_LEFT_DIGITAL)) axis = -1;
				if (worldModel.getKeyBinder().isActionActivated(ACTION_ROTATE_RIGHT_DIGITAL)) axis = 1;
				if (!isNaN(axis)) {
					desiredRotation = MathUtils.map(axis, -1, 1, -MAX_ROTATION, MAX_ROTATION) * MathUtils.DEG2RAD;
				}
			}

			// Finally, applies rotation
			body.applyAngularImpulse((desiredRotation - body.rotation) * LD27.TARGET_FPS * 4);

			// Tries to remain centered
			body.applyAngularImpulse(-body.angularVel * LD27.TARGET_FPS * 2 * (1 - Math.abs(isNaN(axis) ? 0 : axis * 0.9)));

			cameraX -= (cameraX - body.position.x) / 20;
			cameraY -= (cameraY - body.position.y) / 20;

			// Checks if it's close to the ground
			landingRay.origin.set(body.localPointToWorld(landingRayOrigin, true));
			landingRay.direction.set(body.localVectorToWorld(landingRayDirection, true));

			var rayResult:RayResult = worldModel.getSpace().rayCast(landingRay, false, landingFilter);
			if (rayResult != null) {
				// Close to the ground
				if (landingGearExtended != 1) {
					// Need to extend landing gears
					landingGearExtended = Math.min(landingGearExtended + __deltaTimeSeconds / TIME_LANDING_GEAR_CONTRACT_EXTEND, 1);
				}
			} else {
				// Away from the ground
				if (landingGearExtended != 0) {
					// Need to contract landing gear
					landingGearExtended = Math.max(landingGearExtended - __deltaTimeSeconds / TIME_LANDING_GEAR_CONTRACT_EXTEND, 0);
				}
			}

			// Update landing gear weld
			landingGearExtendedEased = Equations.backInOut(landingGearExtended);
			landingGearLeftJoint.phase = MathUtils.fastMap(landingGearExtendedEased, 0, 1, -LANDING_GEAR_ROTATION_RETRACTED, -LANDING_GEAR_ROTATION_EXTENDED);
			landingGearLeftJoint.anchor2.setxy(0, MathUtils.fastMap(landingGearExtendedEased, 0, 1, LANDING_GEAR_POSITION_RETRACTED, LANDING_GEAR_POSITION_EXTENDED));
			landingGearRightJoint.phase = MathUtils.fastMap(landingGearExtendedEased, 0, 1, LANDING_GEAR_ROTATION_RETRACTED, LANDING_GEAR_ROTATION_EXTENDED);
			landingGearRightJoint.anchor2.setxy(0, MathUtils.fastMap(landingGearExtendedEased, 0, 1, LANDING_GEAR_POSITION_RETRACTED, LANDING_GEAR_POSITION_EXTENDED));
		}

		override public function draw(__currentTimeSeconds:Number, __deltaTimeSeconds:Number):void {
			// Called once per visual frame

			// Ship
			sprite.x = body.position.x;
			sprite.y = body.position.y;
			sprite.rotation = body.rotation;

			if (tractorBeamPower == 0) {
				spriteTractorBeam.visible = false;
			} else {
				spriteTractorBeam.visible = true;
				spriteTractorBeam.alpha = tractorBeamPower;
				spriteTractorBeam.x = body.position.x;
				spriteTractorBeam.y = body.position.y;
				spriteTractorBeam.rotation = body.rotation;
			}

			// Landing gears
			if (landingGearExtended == 0) {
				spriteLandingGearLeftUp.visible = spriteLandingGearRightUp.visible = spriteLandingGearLeftDown.visible = spriteLandingGearRightDown.visible = false;
			} else {
				spriteLandingGearLeftUp.visible = spriteLandingGearRightUp.visible = spriteLandingGearLeftDown.visible = spriteLandingGearRightDown.visible = true;

				spriteLandingGearLeftUp.x = landingGearLeft.position.x;
				spriteLandingGearLeftUp.y = landingGearLeft.position.y;
				spriteLandingGearLeftUp.rotation = landingGearLeft.rotation;

				spriteLandingGearRightUp.x = landingGearRight.position.x;
				spriteLandingGearRightUp.y = landingGearRight.position.y;
				spriteLandingGearRightUp.rotation = landingGearRight.rotation;

				spriteLandingGearLeftDown.x = landingGearLeft.position.x;
				spriteLandingGearLeftDown.y = landingGearLeft.position.y;
				spriteLandingGearLeftDown.rotation = landingGearLeft.rotation;

				spriteLandingGearRightDown.x = landingGearRight.position.x;
				spriteLandingGearRightDown.y = landingGearRight.position.y;
				spriteLandingGearRightDown.rotation = landingGearRight.rotation;
			}

			// Set camera

			//cameraX = body.position.x;
			//cameraY = body.position.y;

			// This is not 100% accurate - may miss frames and it look weird?
			worldModel.setCameraPosition(cameraX, cameraY);
		}

		public function addCapturedTarget():void {
			SoundPlayer.playSound(SoundLibrary.CAPTURE);
			_numCapturedTargets++;
		}

		// ================================================================================================================
		// ACCESSOR INTERFACE ---------------------------------------------------------------------------------------------

		public function get fuel():Number {
			return _fuel;
		}

		public function get isOutOfFuel():Boolean {
			return _isOutOfFuel;
		}

		public function get numCapturedTargets():int {
			return _numCapturedTargets;
		}

		public function set numCapturedTargets(numCapturedTargets:int):void {
			_numCapturedTargets = numCapturedTargets;
		}
	}
}
