package com.zehfernando.ld27.display.scenes.game.actors.particles {
	import nape.geom.Vec2;
	import nape.phys.Body;
	import nape.phys.BodyType;
	import nape.shape.Circle;

	import com.zehfernando.ld27.application.LD27;
	import com.zehfernando.ld27.display.scenes.game.actors.Actor;
	import com.zehfernando.ld27.display.scenes.game.world.WorldModel;
	import com.zehfernando.ld27.starling.templates.ShapeSprite;
	import com.zehfernando.utils.MathUtils;

	import flash.filters.BlurFilter;

	/**
	 * @author zeh fernando
	 */
	public class Particle extends Actor {

		// Properties
		private var x:Number;
		private var y:Number;
		private var velocity:Vec2;

		// Properties for speed
		private var f:Number;

		// Instances
		protected var radius:Number;						// In pixels
		protected var timeToLive:Number;
		protected var spriteShape:Class;
		protected var blurRadius:Number;

		protected var alphaStart:Number;
		protected var alphaEnd:Number;
		protected var rotationStart:Number;
		protected var rotationEnd:Number;
		protected var scaleStart:Number;
		protected var scaleEnd:Number;

		private var body:Body;
		private var timeCreated:Number;
		private var sprite:ShapeSprite;


		// ================================================================================================================
		// CONSTRUCTOR ----------------------------------------------------------------------------------------------------

		public function Particle(__x:Number, __y:Number, __velocity:Vec2) {
			super();

			x = __x;
			y = __y;
			velocity = new Vec2(__velocity.x, __velocity.y);

			// Other properties
			timeCreated = NaN;
			radius = 1;
			timeToLive = 1;
			spriteShape = null;
			blurRadius = 0;

			alphaStart = 1;
			alphaEnd = 0;
			rotationStart = 0;
			rotationEnd = 0;
			scaleStart = 1;
			scaleEnd = 1;
		}


		// ================================================================================================================
		// PUBLIC INTERFACE -----------------------------------------------------------------------------------------------

		override public function start():void {

			// Create body
			body = new Body(BodyType.DYNAMIC);
			body.shapes.add(new Circle(radius));
			body.shapes.at(0).material.density = 0.1;
			body.shapes.at(0).material.rollingFriction = 0.5;
			body.shapes.at(0).material.elasticity = 0;
			body.shapes.at(0).filter.collisionGroup = WorldModel.COLLISION_GROUP_PARTICLES;
			body.shapes.at(0).filter.collisionMask = ~(WorldModel.COLLISION_GROUP_PLAYER | WorldModel.COLLISION_GROUP_PARTICLES); // Collides with everything, except the player and particles
			body.disableCCD = true;
			body.allowRotation = false;
			body.position.setxy(x, y);
			body.velocity = velocity;
			body.debugDraw = spriteShape == null || LD27.DEBUG_DRAW_ALL_DEBUG_BODIES;
			worldModel.getSpace().bodies.add(body);

			// Body sprite
			if (spriteShape != null) {
				sprite = blurRadius > 0 ? new ShapeSprite(spriteShape, 1, 1, null, [new BlurFilter(blurRadius, blurRadius, 2), LD27.FILTER_DROP_SHADOW], blurRadius * 2 + LD27.FILTER_DROP_SHADOW.blurX + 1) : new ShapeSprite(spriteShape, 1, 1, null, [LD27.FILTER_DROP_SHADOW], LD27.FILTER_DROP_SHADOW.blurX + 1);
				sprite.rotation = rotationStart;
				sprite.alpha = alphaStart;
				sprite.scaleX = sprite.scaleY = scaleStart;
				worldModel.getWorldView().getParticlesContainer().addChild(sprite);
			}
		}

		override public function stop():void {
			super.stop();

			worldModel.getSpace().bodies.remove(body);
			body = null;

			if (sprite != null) {
				worldModel.getWorldView().getParticlesContainer().removeChild(sprite);
				sprite.dispose();
				sprite = null;
				spriteShape = null;
			}
		}

		override public function update(__currentTimeSeconds:Number, __deltaTimeSeconds:Number):void {
			// Called 60 times per second
			if (isNaN(timeCreated)) timeCreated = __currentTimeSeconds;
			if (__currentTimeSeconds > timeCreated + timeToLive) {
				suicide();
			}
		}

		override public function draw(__currentTimeSeconds:Number, __deltaTimeSeconds:Number):void {
			// Called once per visual frame
			if (sprite != null) {
				f = MathUtils.fastMap(__currentTimeSeconds, timeCreated + timeToLive * 0.5, timeCreated + timeToLive, 0, 1, true);

				sprite.x = body.position.x;
				sprite.y = body.position.y;
				sprite.rotation = MathUtils.fastMap(f, 0, 1, rotationStart, rotationEnd);
				sprite.alpha = MathUtils.fastMap(f, 0, 1, alphaStart, alphaEnd);
				sprite.scaleX = sprite.scaleY = MathUtils.fastMap(f, 0, 1, scaleStart, scaleEnd);
			}
		}
	}
}
