package com.zehfernando.ld27.display.scenes.game.world {
	import com.zehfernando.display.abstracts.ResizableSprite;
	import com.zehfernando.display.components.text.TextSprite;
	import com.zehfernando.display.shapes.Box;
	import com.zehfernando.display.shapes.GradientBox;
	import com.zehfernando.ld27.assets.FontLibrary;
	import com.zehfernando.utils.MathUtils;
	import com.zehfernando.utils.console.log;

	import flash.display.Shape;
	import flash.filters.DropShadowFilter;

	/**
	 * @author zeh fernando
	 */
	public class WorldHUD extends ResizableSprite {
		
		// Constants
		private static const HEIGHT:Number = 80;
		private static const MARGIN:Number = 6;
		private static const PADDING:Number = 10;
		private static const SHAPE_WIDTH:Number = 340;
		private static const FUEL_BAR_STROKE:Number = 1;

		// Properties
		private var fuelCapacity:Number;											// 0 (empty) to 1 (full)
		private var targetPhase:Number;											// 0 (none captured) to 1 (all needed captured)

		// Instances
		private var textFuel:TextSprite;
		private var textTargets:TextSprite;
		private var shapeLeft:Shape;
		private var shapeRight:Shape;
		private var barLeftBackground:Box;
		private var barLeftForeground:GradientBox;
		private var barLeftForegroundMask:Box;
		private var barRightBackground:Box;
		private var barRightForeground:GradientBox;
		private var barRightForegroundMask:Box;

		// ================================================================================================================
		// CONSTRUCTOR ----------------------------------------------------------------------------------------------------

		public function WorldHUD() {
			super();

			// Properties
			_height = HEIGHT;
			cacheAsBitmap = true;
			fuelCapacity = 0;
			targetPhase = 0;

			// Assets

			// Left
			shapeLeft = new Shape();
			shapeLeft.alpha = 0.5;
			addChild(shapeLeft);

			textFuel = new TextSprite(FontLibrary.KUBUS_BOLD, 34, 0xffffff, 1);
			textFuel.text = "FUEL";
			textFuel.filters = [new DropShadowFilter(1, 45, 0x000000, 1, 2, 2)];
			addChild(textFuel);

			barLeftBackground = new Box(100, 100, 0x000000);
			barLeftBackground.alpha = 0.8;
			addChild(barLeftBackground);

			barLeftForegroundMask = new Box();
			addChild(barLeftForegroundMask);

			barLeftForeground = new GradientBox(100, 100, 0, [0xd41c00, 0xd48100, 0xf0e800]);
			barLeftForeground.mask = barLeftForegroundMask;
			addChild(barLeftForeground);

			// Right
			shapeRight = new Shape();
			shapeRight.alpha = 0.5;
			addChild(shapeRight);

			textTargets = new TextSprite(FontLibrary.KUBUS_BOLD, 34, 0xffffff, 1);
			textTargets.text = "HUMANS";
			textTargets.filters = [new DropShadowFilter(1, 45, 0x000000, 1, 2, 2)];
			addChild(textTargets);

			barRightBackground = new Box(100, 100, 0x000000);
			barRightBackground.alpha = 0.8;
			addChild(barRightBackground);

			barRightForegroundMask = new Box();
			addChild(barRightForegroundMask);

			barRightForeground = new GradientBox(100, 100, 0, [0xffff66, 0xffa800]);
			barRightForeground.mask = barRightForegroundMask;
			addChild(barRightForeground);

			updateFuelMask();
			updateTargetMask();
		}


		// ================================================================================================================
		// INTERNAL INTERFACE ---------------------------------------------------------------------------------------------

		override protected function redrawWidth():void {
			shapeLeft.x = MARGIN;
			textFuel.x = MARGIN + PADDING;
			barLeftBackground.x = MARGIN + PADDING;
			barLeftBackground.width = SHAPE_WIDTH - PADDING * 2;
			barLeftForeground.x = MARGIN + PADDING + FUEL_BAR_STROKE;
			barLeftForeground.width = SHAPE_WIDTH - PADDING * 2 - FUEL_BAR_STROKE * 2;
			updateFuelMask();

			barRightBackground.width = SHAPE_WIDTH - PADDING * 2;
			barRightBackground.x = _width - MARGIN - PADDING - barRightBackground.width;
			barRightForeground.width = SHAPE_WIDTH - PADDING * 2 - FUEL_BAR_STROKE * 2;
			barRightForeground.x = _width - MARGIN - PADDING - FUEL_BAR_STROKE - barRightForeground.width;
			updateTargetMask();

			shapeRight.x = _width - MARGIN;
			textTargets.x = _width - MARGIN - PADDING - textTargets.width;
		}

		override protected function redrawHeight():void {
			// Left
			textFuel.y = MARGIN + PADDING;
			shapeLeft.y = MARGIN;

			// Redraw shape left
			shapeLeft.graphics.clear();
			shapeLeft.graphics.beginFill(0x000000, 1);
			shapeLeft.graphics.moveTo(0, 0);
			shapeLeft.graphics.lineTo(textFuel.width, 0);
			shapeLeft.graphics.lineTo(textFuel.width + textFuel.height, textFuel.height);
			shapeLeft.graphics.lineTo(SHAPE_WIDTH, textFuel.height);
			shapeLeft.graphics.lineTo(SHAPE_WIDTH, _height - MARGIN * 2);
			shapeLeft.graphics.lineTo(0, _height - MARGIN * 2);
			shapeLeft.graphics.endFill();

			barLeftBackground.y = MARGIN + PADDING + textFuel.height;
			barLeftBackground.height = _height - MARGIN - PADDING - barLeftBackground.y;
			barLeftForeground.y = MARGIN + PADDING + textFuel.height + FUEL_BAR_STROKE;
			barLeftForeground.height = _height - MARGIN - PADDING - FUEL_BAR_STROKE - barLeftForeground.y;
			updateFuelMask();

			// Right
			textTargets.y = MARGIN + PADDING;
			shapeRight.y = MARGIN;

			// Redraw shape right
			shapeRight.graphics.clear();
			shapeRight.graphics.beginFill(0x000000, 1);
			shapeRight.graphics.moveTo(0, 0);
			shapeRight.graphics.lineTo(-textTargets.width - PADDING, 0);
			shapeRight.graphics.lineTo(-textTargets.width - PADDING - textTargets.height, textTargets.height);
			shapeRight.graphics.lineTo(-SHAPE_WIDTH, textTargets.height);
			shapeRight.graphics.lineTo(-SHAPE_WIDTH, _height - MARGIN * 2);
			shapeRight.graphics.lineTo(0, _height - MARGIN * 2);
			shapeRight.graphics.endFill();

			barRightBackground.y = MARGIN + PADDING + textFuel.height;
			barRightBackground.height = _height - MARGIN - PADDING - barRightBackground.y;
			barRightForeground.y = MARGIN + PADDING + textFuel.height + FUEL_BAR_STROKE;
			barRightForeground.height = _height - MARGIN - PADDING - FUEL_BAR_STROKE - barRightForeground.y;
			updateFuelMask();
		}

		private function updateFuelMask():void {
			// Update the mask of the visible health bar
			barLeftForegroundMask.x = barLeftForeground.x;
			barLeftForegroundMask.y = barLeftForeground.y;
			barLeftForegroundMask.width = Math.round(barLeftForeground.width * fuelCapacity);
			barLeftForegroundMask.height = barLeftForeground.height;
		}

		private function updateTargetMask():void {
			// Update the mask of the target bar
			barRightForegroundMask.y = barRightForeground.y;
			barRightForegroundMask.width = Math.round(barRightForeground.width * targetPhase);
			barRightForegroundMask.x = barRightForeground.x + barRightForeground.width - barRightForegroundMask.width; // Align right
			barRightForegroundMask.height = barRightForeground.height;
		}

		// ================================================================================================================
		// PUBLIC INTERFACE -----------------------------------------------------------------------------------------------

		public function setFuel(__currentFuel:Number, __maxFuel:Number):void {
			// Updates the HUD with the current fuel
			var newFuelCapacity:Number = MathUtils.clamp(__currentFuel/__maxFuel);
			if (fuelCapacity != newFuelCapacity) {
				fuelCapacity = newFuelCapacity;
				updateFuelMask();
			}
		}

		public function setCapturedTargets(__numCapturedTargets:int, __neededTargets:int):void {
			// Updates the HUD with the number of captured targets
			var newTargetPhase:Number = MathUtils.clamp(__numCapturedTargets/__neededTargets);
			if (targetPhase != newTargetPhase) {
				targetPhase = newTargetPhase;
				updateTargetMask();
			}
		}
	}
}
