package com.zehfernando.ld27.display.scenes.game.world {
	import nape.geom.Mat23;
	import nape.space.Space;
	import nape.util.ShapeDebug;

	import starling.core.Starling;
	import starling.display.Sprite;
	import starling.events.Event;

	import com.zehfernando.display.abstracts.ResizableSprite;
	import com.zehfernando.ld27.application.LD27;
	import com.zehfernando.ld27.starling.templates.TextureBank;
	import com.zehfernando.utils.AppUtils;

	import flash.display.Sprite;
	import flash.display3D.Context3DProfile;
	import flash.events.Event;
	/**
	 * @author zeh fernando
	 */
	public class WorldView extends ResizableSprite {

		// Event enums
		public static const EVENT_READY:String = "ready";

		// Properties
		private var worldScale:Number;								// Current view scale (1 = normal)

		// Instances
		private var debugContainer:ShapeDebug;
		private var spriteContainer:flash.display.Sprite;
		private var starling:Starling;
		private var starlingBackgroundContainer:starling.display.Sprite;
		private var starlingParticlesContainer:starling.display.Sprite;
		private var starlingSpritesContainer:starling.display.Sprite;

		// ================================================================================================================
		// CONSTRUCTOR ----------------------------------------------------------------------------------------------------

		public function WorldView() {
			super();

			worldScale = 1;

			// Create debug draw
			if (LD27.DEBUG_DRAW_DEBUG_BODIES_LAYER) {
				debugContainer = new ShapeDebug(AppUtils.getStage().stageWidth, AppUtils.getStage().stageHeight, 0x000000); // Ugh
				debugContainer.thickness = 2;
				//debugContainer.drawCollisionArbiters = !Anathematic.DEBUG_DEMO_MODE;
				debugContainer.drawConstraints = true;
				debugContainer.transform = Mat23.scale(worldScale, worldScale);
				debugContainer.display.alpha = 1;
				addChild(debugContainer.display);
			}

			// Sprite container
			spriteContainer = new flash.display.Sprite();
			spriteContainer.scaleX = spriteContainer.scaleY = worldScale;
			addChild(spriteContainer);

			// Starling instance
			starling = new Starling(starling.display.Sprite, AppUtils.getStage(), null, null, "auto", Context3DProfile.BASELINE);
			starling.antiAliasing = LD27.PLATFORM_ALIASING_LEVEL;
			starling.showStats = LD27.DEBUG_SHOW_STATS_STARLING;
			starling.start();
			starling.addEventListener(starling.events.Event.ROOT_CREATED, onStarlingRootCreated);
		}


		// ================================================================================================================
		// INTERNAL INTERFACE ---------------------------------------------------------------------------------------------

		override protected function redrawWidth():void {
		}

		override protected function redrawHeight():void {
		}


		// ================================================================================================================
		// EVENT INTERFACE ------------------------------------------------------------------------------------------------

		private function onStarlingRootCreated(__e:starling.events.Event):void {
			// Creates containers
			var starlingRoot:starling.display.Sprite = starling.root as starling.display.Sprite;

			starlingRoot.visible = false;

			starlingBackgroundContainer = new starling.display.Sprite();
			starlingRoot.addChild(starlingBackgroundContainer);

			starlingParticlesContainer = new starling.display.Sprite();
			starlingRoot.addChild(starlingParticlesContainer);

			starlingSpritesContainer = new starling.display.Sprite();
			starlingRoot.addChild(starlingSpritesContainer);

			dispatchEvent(new flash.events.Event(EVENT_READY));
			//(Starling.current.root as Sequence).addEventListener(Sequence.EVENT_FINISHED, onStarlingFinished);
		}

		// ================================================================================================================
		// PUBLIC INTERFACE -----------------------------------------------------------------------------------------------

		public function draw(__cameraX:Number, __cameraY:Number, __space:Space):void {
			if (LD27.DEBUG_DRAW_DEBUG_BODIES_LAYER) {
				// Updates debug container
				debugContainer.transform.tx = _width * 0.5 - __cameraX * worldScale;
				debugContainer.transform.ty = _height * 0.5 - __cameraY * worldScale;

				debugContainer.clear();
				debugContainer.draw(__space);
				debugContainer.flush();
			}

			// Updates legacy container
			spriteContainer.x = _width * 0.5 - __cameraX * worldScale;
			spriteContainer.y = _height * 0.5 - __cameraY * worldScale;

			// Updates starling containers
			starling.root.x = _width * 0.5 - __cameraX * worldScale;
			starling.root.y = _height * 0.5 - __cameraY * worldScale;
		}

		public function dispose():void {
			TextureBank.clear();
			starling.dispose();
			starling = null;
		}

		public function getContainerLegacy():flash.display.Sprite {
			return spriteContainer;
		}

		public function getBackgroundContainer():starling.display.Sprite {
			return starlingBackgroundContainer;
		}

		public function getParticlesContainer():starling.display.Sprite {
			return starlingParticlesContainer;
		}

		public function getSpritesContainer():starling.display.Sprite {
			return starlingSpritesContainer;
		}

		public function getStarling():Starling {
			return starling;
		}
	}
}
