package com.zehfernando.ld27.display.scenes.instructions {
	import com.zehfernando.display.components.text.RichTextSprite;
	import com.zehfernando.display.components.text.TextSpriteAlign;
	import com.zehfernando.input.binding.GamepadControls;
	import com.zehfernando.input.binding.KeyActionBinder;
	import com.zehfernando.ld27.assets.FontLibrary;
	import com.zehfernando.ld27.display.scenes.LD27Scene;
	import com.zehfernando.ld27.display.scenes.game.GameScene;
	import com.zehfernando.utils.AppUtils;

	import flash.display.StageDisplayState;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.filters.DropShadowFilter;
	/**
	 * @author zeh fernando
	 */
	public class InstructionsScene extends LD27Scene {

		// Constants
		private static const ACTION_ANY_KEY:String = "anyKey";

		// Instances
		private var textInstructions:RichTextSprite;
		private var binder:KeyActionBinder;


		// ================================================================================================================
		// CONSTRUCTOR ----------------------------------------------------------------------------------------------------

		public function InstructionsScene() {
			super();

			textInstructions = new RichTextSprite(FontLibrary.KUBUS_BOLD, 30, 0xffffff, 1);
			textInstructions.text = "YOU CANNOT BELIEVE YOUR LUCK:\nAFTER A REGULAR MISSION COLLECTING SAMPLES ON EARTH,\nYOU ONLY HAVE 10 SECONDS OF THRUSTER FUEL LEFT!\n\nYOU ONLY HAVE ONE OPTION:\nCAPTURE AT LEAST 10 HUMANS WITH YOUR TRACTOR BEAM.\n(YOUR FUEL IS MADE OF PEOPLE!)\n\n<big>HIJACK HUMANS HASTILY!</big>\n\n-- LAND NEAR HUMANS TO CAPTURE THEM --\n-- USE YOUR KEYBOARD OR GAMEPAD TO CONTROL YOUR SHIP --\n-- DON'T RUN OUT OF THRUSTER FUEL --\n\n(PRESS ANY KEY)";
			textInstructions.blockAlignHorizontal = TextSpriteAlign.CENTER;
			textInstructions.tracking = 2;
			textInstructions.setStyle("big", textInstructions.fontName, 70);
			textInstructions.blockAlignVertical = TextSpriteAlign.MIDDLE;
			textInstructions.align = TextSpriteAlign.CENTER;
			textInstructions.filters = [new DropShadowFilter(1, 45, 0x000000, 1, 2, 2)];
			addChild(textInstructions);

			AppUtils.getStage().addEventListener(MouseEvent.CLICK, onClickGoFullScreen);

			binder = new KeyActionBinder(AppUtils.getStage());
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.BUTTON_ACTION_DOWN);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.BUTTON_ACTION_LEFT);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.BUTTON_ACTION_RIGHT);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.BUTTON_ACTION_UP);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.DPAD_DOWN);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.DPAD_LEFT);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.DPAD_RIGHT);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.DPAD_UP);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.XBOX_START);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.XBOX_BUTTON_ACTION_A);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.XBOX_BUTTON_ACTION_B);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.XBOX_BUTTON_ACTION_X);
			binder.addGamepadActionBinding(ACTION_ANY_KEY, GamepadControls.XBOX_BUTTON_ACTION_Y);
			binder.addKeyboardActionBinding(ACTION_ANY_KEY);
			binder.onActionActivated.add(onAnyKeyPressed);
		}

		// ================================================================================================================
		// INTERNAL INTERFACE ---------------------------------------------------------------------------------------------

		override protected function redrawWidth():void {
			super.redrawWidth();
			textInstructions.x = _width * 0.5;
		}

		override protected function redrawHeight():void {
			super.redrawHeight();
			textInstructions.y = _height * 0.5;
		}

		private function showNextScene():void {
			getSceneViewer().showScene(new GameScene());
		}

		private function onAnyKeyPressed(__action:String):void {
			binder.onActionActivated.removeAll();
			binder.stop();
			AppUtils.getStage().removeEventListener(MouseEvent.CLICK, onClickGoFullScreen);
			showNextScene();
		}

		private function onClickGoFullScreen(__e:Event):void {
			AppUtils.getStage().removeEventListener(MouseEvent.CLICK, onClickGoFullScreen);
			AppUtils.getStage().displayState = StageDisplayState.FULL_SCREEN_INTERACTIVE;
		}


		// ================================================================================================================
		// PUBLIC INTERFACE -----------------------------------------------------------------------------------------------

		override public function dispose():void {
			binder.stop();
			binder = null;
			super.dispose();
		}

	}
}
